% numerical (iteration) and analytical calculation of 
% oscillating frequency of the phase shift oscillator 
clear all; clc;
% CALCULATE RB IF RC, VCC AND BETA ARE KNOWN
VT  = 0.025;
VCC = 9
VBE = 0.65
B1  = 160
RC  = 20000
RB  = RC*(B1+1)*(2*(VCC - VBE)/VCC - 1)
% SET COLLECTOR BIAS VOLTAGE TO VCC/2
IEQ = (VCC - VBE)/(RC + RB/(B1+1))
VC  = VCC - IEQ*RC
% ONCE BIAS IS SET, DETERMINE rpi
ICQ = IEQ*B1/(B1+1);
gm  = ICQ/VT;
rpi = B1/gm
rpi = B1*VT/ICQ
% SET FREQUENCY WITH COMPONENT VALUES
R1  = 10000
R2  = 47000
C1  = 1.0e-06
C2  = 0.47e-06
C3  = 0.33e-06
% DEFINE HELPER RESISTANCES
RxA  = RC*(rpi + RB)/(rpi + RB + RC);
RxB  = rpi*(RC + RB)/(rpi + RB + RC);
RxC  = RB*(rpi + RC)/(rpi + RB + RC);
RRxD = rpi*RB*RC/(rpi + RB + RC);
% CALCULATE FREQUENCY f USING QUADRATIC FORMULA WITH PARAMS A,B and C 
C = -(RxA*C1+RxB*C3);

B = ((RxC-RB)*R1*R2 - (R1+R2)*(R1*RxB+R2*RxA))*C1*C2*C3-...
    (C1*RxA+C3*RxB)*((R1+R2)*(R1+R2)*C2 + (R1*C1*R1+R2*C3*R2))*C2-...
     R2*(R2*RxA+RRxD)*C1*C3*C3 - R1*(R1*RxB+RRxD)*C1*C1*C3;

A = RB*R1*R2*C1*C2*C3*...
    ( (C1*C2+C1*C3+C2*C3)*(R1*R2*(1-RxC/RB) - (R1+R2)*RRxD/RB)+ ...
       C2*(C1*RxA+C3*RxB)*(R1+R2) + C1*C3*(R1*RxB+R2*RxA+RRxD) );

f = sqrt( (-B+sqrt(B*B-4*A*C))/(8*pi*pi*A) )
w = 2*pi*f;
% HAND-CALCULATED DETERMINANTS TO BE COMPARED TO NUMERIC DETERMINANTS LATER
CRAMER_V1 = 1 - w*w*(C1*C2+C1*C3+C2*C3)*R1*R2 + ...
i*(R1*(C1+C2)*w + R2*(C2+C3)*w - RB*R1*R2*C1*C2*C3*w*w*w);

CRAMER_V2 = 1 + RB/rpi - ...
w*w*(R1*R2*(1+RB/rpi)*(C1*C2+C1*C3+C2*C3) + C1*C3*RB*R1 + C2*C3*RB*(R1+R2))+... 
i*w*(R1*(1+RB/rpi)*(C1+C2) + R2*(1+RB/rpi)*(C2+C3) + C3*RB)-...
i*w*w*w*R1*R2*RB*C1*C2*C3;

DET_Y     = 1 -... 
w*w*(C1*C2*(R1*R2+(R1+R2)*RxA))-... 
w*w*(C1*C3*(R1*R2+R1*RxB+R2*RxA+RRxD) + C2*C3*(R1*R2+(R1+R2)*RxB))+...
i*w*(C1*(R1+RxA) + C2*(R1+R2) + C3*(R2+RxB)) -... 
i*w*w*w*C1*C2*C3*((R1+R2)*RRxD + R1*R2*RxC);

% PRINT RESULTS TO CONSOLE
B_NEEDED = -(RB*rpi/RRxD)*...
( 1 - w*w*(C1*C2*(R1*R2+(R1+R2)*RxA) + C1*C3*(R1*R2+R1*RxB+R2*RxA+RRxD)+...
C2*C3*(R1*R2+(R1+R2)*RxB)) ) / (1 - R1*R2*(C1*C2+C1*C3+C2*C3)*w*w)

GAIN     = B1*RRxD*CRAMER_V2/(rpi*RB*DET_Y)
GAIN_ABS = abs(GAIN)
NODE_V1  = -(RRxD/RB)*CRAMER_V1/DET_Y
NODE_V2  = -(RRxD/RB)*CRAMER_V2/DET_Y

% INCREASE FREQUENCY SLIGHTLY AND LOOP DOWN UNTIL IMAGINARY PART CHANGES SIGN
% EVALUATE DETERMINANTS ON EACH ITERATION
% THIS IS FOR VERIFYING CORRECTNESS OF MANUAL CALCULATIONS
f = f + 0.05; angled = -1;
while angled < 0
f = f - 0.00001;
w = 2*pi*f;

Y11 = 1/rpi + 1/RB + i*w*C3;
Y12 = Y21 = 1/RB;
Y14 = Y41 = i*w*C3;
Y22 = 1/RC + 1/RB + i*w*C1;
Y23 = Y32 = i*w*C1;
Y33 = 1/R1 + i*w*C1 + i*w*C2;
Y34 = Y43 = i*w*C2;
Y44 = 1/R2 + i*w*C2 + i*w*C3;
% V1/V1
CRAMER_V1=[ -Y12    0     -Y14 ; ...
            -Y32    Y33   -Y34 ; ...
             0     -Y43    Y44 ];
% V2/V1
CRAMER_V2=[  Y11    0     -Y14 ; ...
             0      Y33   -Y34 ; ...
            -Y41   -Y43    Y44 ];
% Common denominator
DET_Y=[ Y11  -Y12   0    -Y14  ; ...
       -Y21   Y22  -Y23   0    ; ...
        0    -Y32   Y33  -Y34  ; ...
       -Y41   0    -Y43   Y44  ];
 
angled = arg(det(CRAMER_V1)/det(DET_Y));
end; % while
% PRINT RESULTS TO CONSOLE
f
B_NEEDED = (det(DET_Y)/det(CRAMER_V1))*rpi
GAIN     = (B1/rpi)*det(CRAMER_V2)/det(DET_Y)
GAIN_ABS = abs(GAIN)
NODE_V1  = det(CRAMER_V1)/det(DET_Y)
NODE_V2  = -det(CRAMER_V2)/det(DET_Y)

